#!/bin/bash
#filename: isadep.sh
#Author: Andy Laughton
#Version: 0.1
#Date: 20190304
#Description: this script checks if a package is a dependency for an installed package.
#It should be possible to integrate this with other fronty scripts to determine whether 
#a package can be deleted or not.
#Need to check a package was passed to the script

#Exit codes
#0 - package is not a dependency for another package, in other words, succes. It can be deleted without issues.  Probably.
#Optional packages need to be added to the category.apps.* files
#1 - package is needed by another package
#3 - unused, for future use, Thargons maybe...
#4 - package name not passed to the script
#5 - /etc/fronty/fronty.conf does not exist
#6 - package does not exist


if [ -z $1 ]; then
	echo "Usage isadep.sh packagename.  Exiting to Witch Space..."
	exit 4
fi

#Get variables from fronty.conf if it exists otherwise exit.
if [ ! -f /etc/fronty/fronty.conf ]; then
	echo "/etc/fronty/fronty.conf missing!  Exiting..."
	exit 5
fi
source /etc/fronty/fronty.conf

#Set dependency to zero
dependency=0
#Set pkg to $1
pkg=$1
#Check the package exists
pkg_exists=$(grep ,$pkg, $config/category.apps.index | head -1 | cut -f2 -d,) 1>/dev/null
if [ -z $pkg_exists ]; then
	echo "Package "$pkg" doesn't exist, or it is a typo.  Exiting to battle Thargoids..."
	exit 6
fi

#Now check if $pkg is a dependency for any installed packages
for installedpkg in $(ls /var/log/packages | grep SBo | rev | cut -f4- -d- | rev); do 
	#echo $installedpkg
	grep ,$installedpkg, $config/category.apps.deps | cut -f4 -d, | grep $pkg 1>/dev/null
	if [ $? = 0 ]; then
		echo $installedpkg
		dependency=1
	fi
done
exit $dependency
