#!/bin/bash
#This is the update script for packages
#Filename update_package.sh
#Date 20180819
#Version 0.2
#Author Andy Laughton
 
# Exit codes
# 0 - worked successfully
# 1 - config file missing
# 2 - error with 64 bit source download
# 3 - error with 32 bit source download
# 4 - SlackBuild files not available
# 5 - compilation failed
# 6 - script already running - code to be added - flag file perhaps?

# Version 0.2
# Added functionality to check if the latest version of the package is installed
# If not the script does an upgrade.  If the package is not installed it does an install.

#Check an app was passed to the script
if [ -z $1 ]; then
	echo "Package name missing, exiting"
	exit 1
fi

#Initialise all variables to blank.
progs=""
progcat=""
progparams=""
category=""
prgnam=""
version=""
homepage=""
dwnload=""
md5sum=""
dwnload_x86_64=""
md5sum_x86_64=""
dwnld32=""
dwnld64=""
build=""
pkgtar=""
pkgoptions=""
prg=""
pkg=""
pkgname=""
options=""
compiledate=""
pkg_found=""

#Get the values for dwnlddir and slackver - if the config file is missing, print
#an error and exit
if [ ! -f /etc/fronty/fronty.conf ]; then
	echo "/etc/fronty/fronty.conf missing!  Exiting..."
	exit 1
fi
source /etc/fronty/fronty.conf

#Export the variables so pre- and post- scripts have them if needed
export source=$source
export config=$config
export scripts=$scripts
export slackver=$slackver
export logfile=$fronty_logfile

#progs variable lists the package to be upgraded.
progs=$1

#Check category.apps.index and category.apps.data are available, if not print an error and run the script.
if [ ! -f $config/category.apps.index ] || [ ! -f $config/category.apps.data ]; then
	echo "A category index or data file is missing, running script..."
	$basedir/build_category_apps_repo.sh
fi

echo "Start build of $progs">> $fronty_logfile
# Check if the package exists.  If not it may be a typo and progs is incorrect.
# Exit if incorrect

#Change this, get rid of var, not really needed
pkg_exists=$(grep ,$progs, $config/category.apps.index | head -1 | cut -f2 -d,)
if [ -z $pkg_exists ]; then
	echo "Package "$progs" doesn't exist, or it is a typo."
	echo "Please fix and rerun.  Exiting"
	echo "Package "$progs" doesn't exist, or it is a typo." >> $fronty_logfile
	exit 7
fi

#Check the package is not in /etc/fronty/fronty_blacklist which lists
#packages that shouldn't be autoupdated
#Need to tweak this...
#if [ $(grep $progs /etc/fronty/fronty_blacklist | grep -v \# | head -1) ]; then
#	echo "Autoupdate of "$progs" not allowed, run manual update.  Exiting.." >> $fronty_logfile
#	echo "Autoupdate of "$progs" not allowed, run manual update.  Exiting.."
#	exit 0
#fi

#Now check if it has a directory, if not get the download info and Slackbuild files
#from category.apps.data, download, check md5sum and continue.
echo "Date :" `date +%Y%m%d-%H%M%S` " Start package : " $progs >> $fronty_logfile

#Fields in category.apps.data:
#category,$prgnam,$version,$homepage,$download,$md5sum,$dwnld_x86_64,$md5sum_x86_64,$requires
#Get the pkg info and split it into separate variables
progcat=$(grep ","$progs"," $config/category.apps.index)
progparams=$(grep $progcat $config/category.apps.data)
category=$(echo $progparams | cut -f1 -d,)
version=$(echo $progparams | cut -f3 -d,)
dwnld=$(echo $progparams | cut -f5 -d,)
md5sum=$(echo $progparams | cut -f6 -d,)
dwnld_x86_64=$(echo $progparams | cut -f7 -d,)
md5sum_x86_64=$(echo $progparams | cut -f8 -d,)
echo "progparams : "$progparams >> $fronty_logfile

#cd to the source directory
cd $source
pkg_found=$(ls /var/log/packages | grep $progs | head -1)
if [ $(echo $pkg_found | wc -c) -gt 1 ] && [ $(echo $pkg_found | rev | cut -f3 -d- | rev) <> $version ] || [ $(echo $pkg_found | wc -c) -lt 3 ]; then
	echo "Older, or no version of package "$progs" installed" >> $fronty_logfile
	if [ ! -d $progs ]; then 
		#Create the dir with the progs as its name
		mkdir $progs
		echo "Created dir for "$progs >> $fronty_logfile
		cd $progs
		#Create the dir with the version as its name
		mkdir $version
		echo "Created dir version "$version" for "$progs >> $fronty_logfile
		cd $version
	else
		cd $progs
		if [ -d $version ]; then 
			cd $version
		else
			mkdir $version
			cd $version
		fi
	fi
	# Now for some processing of the downloads and md5sums - some packages have more than one download
	# and because of this the downloads have to be in a file.  Variables cannot hold newline characters.
	# If a variable has spaces in it it causes issues with the -z option (if [ -z varname ...) because
	# only one argument is expected, whereas having a space means there is more than one argument which
	# is why wc -c is used.
	# 64 bit downloads
	if [ `echo $dwnld_x86_64 | wc -c` -gt 1 ]; then
		echo $dwnld_x86_64 | sed s/\ /\\n/g > link64
		echo $dwnld_x86_64 | sed s/\ /\\n/g | rev | cut -f1 -d\/ | rev > file64
		echo $md5sum_x86_64 | sed s/\ /\\n/g > md5sum64-tmp
		paste md5sum64-tmp file64 | sed s/\\t/"  "/g > md5sum64
	elif [ `echo $dwnld | wc -c` -gt 1 ]; then
		echo $dwnld | sed s/\ /\\n/g > link32
		echo $dwnld | sed s/\ /\\n/g | rev | cut -f1 -d\/ | rev > file32
		echo $md5sum | sed s/\ /\\n/g > md5sum32-tmp
		paste md5sum32-tmp file32 | sed s/\\t/"  "/g > md5sum32
	fi

	# If the source has already been downloaded don't download it again.
	# check the machine arch first then check if there is a 64 bit or 32 bit download
	if [ `uname -m` == "x86_64" ]; then
		#Is there an x86_64 download, if so download it
		if [ -f file64 ]; then
			#If the file is already here don't download it again.
			for dwnld64 in $(cat file64); do
				if [ ! -f $dwnld64 ]; then
					echo "Fetching x86_64 download "$dwnld64 >> $fronty_logfile
					wget $(grep $dwnld64 link64)
					echo "Download complete for "$dwnld64 >> $fronty_logfile
				fi
				grep $dwnld64 md5sum64 > "md5sum64-"$progs
				md5sum -c "md5sum64-"$progs
				if [ $? != 0 ]; then
					echo "Error with x86_64 downloaded source for " $progs" exiting..." >> $fronty_logfile
					echo >> $fronty_logfile
					exit 2
				fi
				echo "md5sum_x86_64 OK for "$dwnld_x86_64 >> $fronty_logfile
			done
		# There may be a 32 bit download 
		elif [ -f file32 ]; then
			#If the file is already here don't download it again.
			for dwnld32 in $(cat file32); do
				if [ ! -f $dwnld32 ]; then
					echo "Fetching x86 download "$dwnld32 >> $fronty_logfile
					wget $(grep $dwnld32 link32)
					echo "Download complete for "$dwnld32 >> $fronty_logfile
				fi
				grep $dwnld32 md5sum32 > "md5sum32-"$progs
				md5sum -c "md5sum32-"$progs
				if [ $? != 0 ]; then
					echo "Error with downloaded source for " $progs" exiting..." >> $fronty_logfile
					echo >> $fronty_logfile
					exit 3
				fi
				echo "md5sum OK for "$dwnld32 >> $fronty_logfile
			done
		fi
	elif [ `uname -m` == "x86" ] || [ $(uname -m) == "i686" ]; then
		#Download the x86 source
		for dwnld32 in $(cat file32); do
			if [ ! -f $dwnld32 ]; then
				echo "Fetching x86 download "$dwnld32 >> $fronty_logfile
				wget $(grep $dwnld32 link32)
				echo "Download complete for "$dwnld32 >> $fronty_logfile
			fi
			grep $dwnld32 md5sum32 > "md5sum32-"$progs
			md5sum -c "md5sum32-"$progs
			if [ $? != 0 ]; then
				echo "Error with downloaded source for " $progs" exiting..." >> $fronty_logfile
				echo >> $fronty_logfile
				exit 3
			fi
			echo "md5sum OK for "$dwnld32 >> $fronty_logfile
		done
	fi
	# Check the SlackBuild files are available and copy to the source dir
	if [ ! $config/$category/$PRG/$progs".SlackBuild" ]; then
		echo $progs".SlackBuild not available, exiting..." >> $fronty_logfile
		echo >> $fronty_logfile
		exit 4
	else	
		cp -R $config/$category/$progs/* .
		echo "Copied "$progs" SlackBuild files...">>$fronty_logfile
		#build not really needed so commented out to make sure before I delete it.
		#build=$(grep ^BUILD $progs".SlackBuild" | cut -f2 -d- | rev | cut -b2- | rev)
	fi

	#If there are any prebuild scripts, run them now.
	if [ -f $config/$progs.preoptions ]; then
		echo "Running prebuild script "$progs".preoptions now..>" >> $fronty_logfile
		$config/$progs.preoptions
	fi
	#Run the Slackbuild script
	#Note that some packages may require extra options so add them here in $pkgoptions,
	#from category.apps.options
	#If the SBo package is not present compile, otherwise to the install
	if [ ! -f $progs"-"$version*"SBo"* ]; then
		compiledate=$(date +%Y%m%d-%H%M%S)
		if [ -f $config/$progs.options ]; then
			echo "Compiling with options..." >> $fronty_logfile
			echo >> $fronty_logfile
			source $config/$progs.options
			sh ./$progs.SlackBuild > ./$progs"_"$compiledate"_build.log"
		else
			echo "Compiling without options..." >> $fronty_logfile
			sh ./$progs.SlackBuild > ./$progs"_"$compiledate"_build.log"
		fi
	
		if [ ! `tail -2 $progs"_"$compiledate"_build.log"|head -1 | rev | cut -f1 -d" "| rev | grep created` ]; then
		 	echo "Compilation of "$progs" failed, check "$progs"_"$compiledate"_build.log and "$fronty_logfile >> ./$progs"_"$compiledate"_build.log"
			echo "Compilation of "$progs" failed, check "$progs"_"$compiledate"_build.log and "$fronty_logfile " "$(date +%Y%m%d-%H%M%S) >> $fronty_logfile
			echo >> $fronty_logfile
			#This should exit but it doesn't seem to work
			exit 5
		fi
		pkg=$(tail -2 ./$progs"_"$compiledate"_build.log" | head -1 | cut -b19- | rev | cut -b10- | rev)
		mv $pkg .
		echo $pkg" moved from /tmp to "$(pwd)
	else
		# Package already compiled
		echo $progs" already compiled, SBo exists..." >> $fronty_logfile
	fi
	#If the compile was skipped, an SBo file is present but the var pkg has not been set, so pkgname 
	#will not be set.  Do it here.
	if [ ! -z $pkg ]; then
		pkgname=$(echo $pkg | cut -b6-)
	else
		pkgname=$( ls | grep SBo)
	fi
	echo $pkgname
	#If the package isn't installed then just do a straight install otherwise upgrade
	#This will only work if there are no missing dependencies
	if [ ! $(ls /var/log/packages|grep $pkg_found | rev | cut -f4 -d- | rev) ]; then
		echo "Installing package "$pkgname >> $fronty_logfile
		installpkg $pkgname
	else
		echo "Upgrading package "$pkgname >> $fronty_logfile
		upgradepkg $pkgname
		#Delete the old zero byte file showing the version
		rm $source/$(echo $pkg_found | rev | cut -f3- -d- | rev)
	fi
	#Add the zero byte file showing the package and version number
	touch $source/$progs-$version
	# Run any postoptions scripts		
	if [ -f $config/$progs.postoptions ]; then
		echo "Running postoptions script..." >> $fronty_logfile
		echo >> $fronty_logfile
		$config/$progs.postoptions
	fi
	echo >> $fronty_logfile
else
	echo "pkg installed"
	echo "Package "$progs" already installed, skipping..." >> $fronty_logfile
	echo >> $fronty_logfile
fi
# Set all  the variables to blank.  Good practice.
progs=""
progcat=""
progparams=""
category=""
PRGNAM=""
version=""
homepage=""
dwnload=""
md5sum=""
dwnload_x86_64=""
md5sum_x86_64=""
dwnld32=""
dwnld64=""
build=""
pkgtar=""
pkgoptions=""
prg=""
pkg=""
pkgname=""
options=""
compiledate=""
echo "Date :" `date +%Y%m%d-%H%M%S` " Finished $progs" >> $fronty_logfile
echo "================================================================================" >> $fronty_logfile
echo >> $fronty_logfile
