#!/bin/bash
# Author Andy Laughton
# Date 20170116
# Script to update build scripts with the latest script template.
# Get the line starting PROGS= with it's line number and drop into a tmp file
# copy the same number of lines -1 from the new template to the build script being updated
# Copy the PROGS= line from the tmp file to the end of the new build script
# Copy the rest of the build script template (lines+1) to the new updated build script
# Copy the lines above #Author (usually the fifth line) to another file, then copy back these
# first followed by the rest.  This means any comments at the top are retained.
# After the new files are created they need to be set to executable as they are new files
# and perms are set to make new files not executable.  So this will only work on executable
# scripts.  If you wish to change this then change the for SCRIPTNAME line
# Get the number of the line where PROGS= is in the build template

# Set global vars
# The number of the line where progs is defined
numlines=$(grep -n progs= "build_template" | cut -f1 -d:)
# The number of lines in the build_template
lastline=$(wc -l "build_template" | cut -f1 -d" ")
# The number of lines below the line defining PROGS in build_template
taillines=$(expr $lastline - $numlines)

#The line below is here for testing.  Uncomment it and comment the one below it
#when testing new changes.
#for scriptname in $(ls | grep build_wireshark.sh); do
for scriptname in $(ls -l build_*.sh | grep ^-rwx | rev | cut -f1 -d" " | rev); do
	# Get the number of lines down to and including the Author
	toplines=$(grep -n \#Author $scriptname | cut -f1 -d:)
	# Copy the top lines above the author to $scriptname.toplines
	head -n $(expr $toplines - 1) $scriptname > $scriptname.toplines
	# Find the line with the progs variable and copy to $scriptname.progsline
	# Added -i as the current update drops all vars to lowercase and this won't work without it
	grep -i progs= $scriptname > $scriptname".progsline"
	#Change PROGS to lowercase
	sed -i s/PROGS/progs/ $scriptname.progsline
	# Cat the contents of $scriptname.toplines into the new $scriptname
	cat $scriptname.toplines > $scriptname
	# Copy the lines above the progs= line in the build_template except the lines above the date to $scriptname
	head -n $(expr $numlines - 1) build_template | tail -n $(expr $numlines - $toplines) >> $scriptname
	# Cat the contents of $scriptname.progsline to the new $scriptname
	cat $scriptname.progsline | cut -f2 -d: >> $scriptname
	tail -n $(expr $lastline - $numlines) "build_template" >> $scriptname
	chmod u+x $scriptname
	rm $scriptname".progsline"
	rm $scriptname".toplines"
done
