#!/bin/bash
#Script to sync the local slackbuild cache with files on slackbuilds.org
#Filename update_repo.sh
#Date 20180329
#Author Andy Laughton
#Version 0.2
#V0.1 - first version that worked :-)
#V0.2 - added code to generate a deps file - category.apps.predeps working out all dependencies for 
#all packages that have them.
#V0.3 - changed name and removed the command to run $scripts/get_alldeps.sh - run after checking 
#proj.info is properly formatted. The maintainer is adding double quotes to the end of the first line
#of the md5sums instead of a backslash which screws up $scripts/get_alldeps.sh - any package with proj
#as a dependency generates an infinite number of dependency files so ctrl-C is needed.
#Date: 20200302
#V0.3.1 - merged get_alldeps.sh with this file then renamed to update_repo.sh

#Check if the config file fronty.conf exists, if not print info on what should be set and exit
#If you want your path to be different then edit fronty.conf accordingly and change the values 
#listed here.  If you want files for a different version of Slackware just change the value in fronty.conf.
#20190413
#All vars changed to lowercase
#Todo

#If an upgrade has been done we need to detect this and move source etc as usual but the new dir name will be the new version.
#Or we can skip the Slackware version in the path.  for a future release along with the installation script which will, for now,
#ensure the installed version of Slackware is used in the path.
if [ ! /etc/fronty/fronty.conf ]; then
	echo "Config file /etc/fronty/fronty.conf does not exist so probably not installed!"
	echo "Please set the vars slackver, basedir, config, scripts, source and fronty_logfile changing"
	echo "the paths as required"
	echo 'slackver=$(cat /etc/slackware-version|cut -f2 -d" ")'
	echo "basedir=/data/sborepo - for build_category_apps_repo.sh"
	echo "config=/data/sborepo/"$slackver"/config - for sbo files and category.* files"
	echo "scripts=/data/sborepo/"$slackver"/scripts - for the build scripts"
	echo "source=/data/sborepo/"$slacker"/source - for the source files"
	echo "autoupdate=1 or 0 - determines if packages with newer versions are updated or not"
	echo "fronty_logfile=/var/log/fronty.log"
	exit 1
else
	#Get the variables and their values from fronty.conf
	source /etc/fronty/fronty.conf
	#Check the dirs exist by checking if the config dir exists, if not create them, write to logfile.  If
	#any dir exists it should be config as that holds the category.* files and the Slackbuild files.
	#Need to add an installation script which will ask for these values.
	if [ ! -d $config ]; then
		echo `date +%Y%m%d-%H%M%S`" "$basedir/$slackver" does not exist, creating it, plus subdirs" >> $fronty_logfile
		mkdir -p $config $scripts $source
		echo `date +%Y%m%d-%H%M%S`" "$basedir/$slackver"/{config,scripts,source} created..." >> $fronty_logfile
		echo `date +%Y%m%d-%H%M%S`" Running rsync..." >> $fronty_logfile
		rsync -a rsync://slackbuilds.org/slackbuilds/$slackver/$config/
		echo `date +%Y%m%d-%H%M%S`" rsync complete ..." >> $fronty_logfile

	else
		#Dirs do exist, backup by appending the current date and time to the current dir and create a new one
		#Set dateandtime to the current date and time
		dateandtime=$(date +%Y%m%d-%H%M%S)
		#Rename the current repo dir by appending current date and time to the name
		mv $basedir/$slackver $basedir/$slackver-$dateandtime
		#Create new repo dir, adding config, script and source dirs
		mkdir -p $config $scripts $source
		echo $dateandtime" previous sborepo renamed to 14.2-"$dateandtime", new version "$slackver" created..." >> $fronty_logfile

		#If any package options files exist copy them to the new dir
		#Note - the options required may change with different versions of 
		#packages requiring different options
		echo "Copying pkg options files to new sborepo..." >> $fronty_logfile
		cp -a $basedir/$slackver-$dateandtime/config/*options $config/

		#Run rsync
		echo `date +%Y%m%d-%H%M%S`" Running rsync ..." >> $fronty_logfile
		rsync -a rsync://slackbuilds.org/slackbuilds/$slackver/ $config/
		echo `date +%Y%m%d-%H%M%S`" rsync complete ..." >> $fronty_logfile

		echo "Copying scripts contents..." >> $fronty_logfile
		echo "Check if any scripts need changing..." >> $fronty_logfile
		#If any scripts exist copy them to the new scripts dir
		cp -a $basedir/$slackver-$dateandtime/scripts/* $scripts/
		echo "Scripts copied..." >> $fronty_logfile
		#Move the source dirs from $basedir/$slackver-$dateandtime/source to the new $source
		#If there is zero byte file for a package that isn't installed then delete it
		cd $basedir/$slackver-$dateandtime/source
		#for a in $(ls -l | grep -v ^dr | grep -v ^total | grep -v updated_source | rev | cut -f1 -d" "); do
		#	pkgname=$(echo $a | cut -f2- -d- | rev)
		#	pkgver=$(echo $a | cut -f1 -d- | rev)
		#	#echo "pkgname="$pkgname
		#	#echo "pkgver="$pkgver
		#done
		for sourcepkg in $(ls -l | grep ^dr | rev | cut -f1 -d" " | cut -b1- | rev); do
			installedversion=$(ls /var/log/packages | grep ^$sourcepkg | head -1 | rev | cut -f3 -d- | rev)
			#newversion=$(grep ,$sourcepkg, ../config/category.apps.data | cut -f3 -d,)
			mv $sourcepkg $source/
			if [ -f $sourcepkg"-"[0-9a-zA-Z] ]; then
				mv $sourcepkg"-"* $source
			elif [ -f $sourcepkg"-"$newversion ]; then
				mv $sourcepkg"-"$newversion $source
			fi
			echo "Version "$installedversion" of "$sourcepkg" moved to new source...">> $fronty_logfile
			#echo $sourcepkg,$installedversion,$newversion
			#if [ $(echo $installedversion | wc -c) -gt 1 ]; then
			#	echo "installedversion = "$installedversion
			#	echo "newversion = "$newversion
			#	if [ $newversion != $installedversion ]; then
			#		echo $sourcepkg,$installedversion,$newversion  >> $source/updated_source.log
			#	fi
			#fi
		done
		echo $dateandtime" category.apps.index and category.apps.data created/updated...">>$fronty_logfile
	fi
fi
echo $(date +%Y%m%d-%H%M%S)" Finished rsync..." >> $fronty_logfile

cd $config

#Create a text file containing a list of the categories.  If there is an existing one rename it by 
#appending the current date and time to it's name.
if [ -f category ]; then
	mv category category.$dateandtime
	echo $dateandtime" Category file renamed..." >> $fronty_logfile
fi
ls -l | grep ^dr | grep -v tmpwork | rev | cut -f1 -d" " | rev > category
echo $(date +%Y%m%d-%H%M%S)" New category file created..." >> $fronty_logfile

for category in $(cat category); do
	cd $category
	for pkg in $(ls -l | grep ^dr | rev | cut -f1 -d" " | rev); do
        	cd $pkg
	        source $pkg.info
		echo $category,$PRGNAM,$VERSION>>$config/category.apps.index
		echo $category,$PRGNAM,$VERSION,$REQUIRES>>$config/category.apps.predeps
		echo $category,$PRGNAM,$VERSION,$HOMEPAGE,$DOWNLOAD,$MD5SUM,$DOWNLOAD_x86_64,$MD5SUM_x86_64,$REQUIRES>>$config/category.apps.data
		cd ..
	done
	cd ..
done

#Now change all %README%s to blank as it messes up the script that sorts out dependencies and put in a separate file
sed -i s/\%README\%//g $config/category.apps.predeps

#tar up the previous sborepo dir
cd $basedir
tar czf $slackver-$dateandtime.tgz $slackver-$dateandtime && rm -R $slackver-$dateandtime

#Next step is to generate all the dependencies
echo  >> $fronty_logfile
echo "Now run "$scripts"/get_alldeps.sh" >> $fronty_logfile

# Now to change all the UNSUPPORTEDs to blanks which will fix an issue with the build template
#sed s/UNSUPPORTED//g category.apps.data > category.apps.data.fixed
#
#Now get all the dependencies
#Get the dependencies for all packages and add to $config/category.apps.dep
#I changed $basedir/build_category.sh script to add an extra file called $config/category.apps.predeps
#This has %README% removed where it occurs.  There are only about a dozen and it only
#occurs in the dependencies field.  This script uses category.apps.predeps to get it's info 
#then drops it into category/apps.deps
#Check the fronty config file exists, if not print an error and exit.
if [ ! -f /etc/fronty/fronty.conf ]; then
	echo "/etc/fronty/fronty.conf missing!  Exiting..."
	exit 1
fi
source /etc/fronty/fronty.conf

#Cd to the working dir for this
basedir=/tmp/config
if [ ! -d $basedir ]; then
	mkdir -p $basedir
fi
cd $basedir

#If there is already a file called category.apps.deps then rename it by appending the 
#date and time to it
if [ -f $config/category.apps.deps ]; then
	mv $config/category.apps.deps $config/category.apps.deps_$(date +%Y%m%d-%H%M%S)
fi

while read pkginfo; do
	#Sets the deps level needed for the package
	lvl=1
	pkgcat=$(echo $pkginfo | cut -f1 -d,)
	pkgname=$(echo $pkginfo | cut -f2 -d,)
	pkgversion=$(echo $pkginfo | cut -f3 -d,)
	pkgdeps=$(echo $pkginfo | cut -f4 -d,)
	touch $pkgname.deps
	if [ $(echo $pkgdeps | wc -c) -gt 2 ]; then
		echo $pkgdeps > $pkgname.$lvl
		#Remove blank lines
		sed -i '/^\s*$/d' $pkgname.$lvl
		#While there are dependencies we keep checking
		deps=1
		while [ $deps != 0 ]; do
			for pkg in $(cat $pkgname.$lvl); do
				grep ,$pkg, $config/category.apps.predeps | cut -f4 -d, >> $pkgname.$(($lvl+1));
			done;
			#Remove blank lines
			sed -i '/^\s*$/d' $pkgname.$(($lvl+1));
			#Change line feeds to spaces
			cat $pkgname.$(($lvl+1)) | tr '\n' ' ' > $pkgname.$(($lvl+1)).tmp
			mv $pkgname.$(($lvl+1)).tmp $pkgname.$(($lvl+1))
			#If there are more deps increment lvl
			if [ $(cat $pkgname.$(($lvl+1)) | wc -c) -gt 1 ]; then
				((lvl++))
			else
				#deps=0
				break
			fi
		done
		#Now we need to add the deps together starting with the last ones
		while [ $lvl != 0 ]; do
			for pkg in $(cat $pkgname.$lvl); do
				grep $pkg $pkgname.deps 1>/dev/null
				if [ $? != 0 ]; then
					echo $pkg >> $pkgname.deps
				fi
			done
			((lvl--))
		done
		cat $pkgname.deps | tr '\n' ' ' > $pkgname.deps.tmp
		mv $pkgname.deps.tmp $pkgname.deps
	fi
	echo $pkgcat,$pkgname,$pkgversion,$(cat $pkgname.deps) >> $config/category.apps.deps
	rm $basedir/$pkgname*
	lvl=1
	lvlvount=""
	lvlfiles=""
	pkgcat=""
	pkgname=""
	pkgversion=""
	pkginfo=""
	pkgdeps=""
	deps=0
done < $config/category.apps.predeps
